{
module Exp where
import Char
import Regexp
}
%name calc
%tokentype { Token }
%token 
      let             { TokenLet }
      in              { TokenIn }
      int             { TokenFloat $$ }
      string	      { TokenString $$ }
      var             { TokenVar $$ }
      '='             { TokenEq }
      '<'             { TokenLT }
      '<='            { TokenLEQ }
      '>'             { TokenGT }
      '>='            { TokenGEQ }
      '+'             { TokenPlus }
      '++'	      { TokenAppend }
      '-'             { TokenMinus }
      '*'             { TokenTimes }
      '/'             { TokenDiv }
      '('             { TokenOB }
      ')'             { TokenCB }
%%
Exp : ExpNum		{ $1 }
    | ExpStr		{ $1 }

ExpNum  : let var '=' int in ExpNum   { \p -> $6 (($2,(show $4)):p) }
        | ExpB                        { $1 }

ExpStr : let var '=' string in ExpStr { \p -> $6 (($2,$4):p) }
       | ExpS			      { $1 }

ExpS : Elem '=' Elem		{ \p -> $1 p == $3 p }
     | Elem '=' Elem '++' Elem	{ \p -> $1 p == (($3 p)++($5 p)) }
     | Elem '<' Elem		{ \p -> $1 p < $3 p }
     | Elem '<=' Elem		{ \p -> $1 p <= $3 p }
     | Elem '>' Elem		{ \p -> $1 p > $3 p }
     | Elem '>=' Elem		{ \p -> $1 p >= $3 p }

Elem : var			{ \p -> case find $1 p of
	                                    Nothing -> error "no var"
					    Just i  -> i }

ExpB  : Exp1 '=' Exp1		{ \p -> $1 p == $3 p }
      | Exp1 '<' Exp1		{ \p -> $1 p < $3 p }
      | Exp1 '<=' Exp1		{ \p -> $1 p <= $3 p }
      | Exp1 '>' Exp1		{ \p -> $1 p > $3 p }
      | Exp1 '>=' Exp1		{ \p -> $1 p >= $3 p }

Exp1  : Exp1 '+' Term           { \p -> $1 p + $3 p }
      | Exp1 '-' Term           { \p -> $1 p - $3 p }
      | Term                    { $1 }

Term  : Term '*' Factor         { \p -> $1 p * $3 p }
      | Term '/' Factor         { \p -> $1 p / $3 p }
      | Factor                  { $1 }

Factor			  
      : int                     { \p -> $1 }
      | var                     { \p -> case find $1 p of
	                                    Nothing -> error "no var"
					    Just i  -> (read i)::Float }
      | '(' Exp1 ')'             { $2 }
{
happyError :: [Token] -> a
happyError _ = error "Parse error"

find::String->[(String,String)]-> (Maybe String)
find e [] = Nothing
find e (x:xs)
	|e==n = (Just v)
	|otherwise = find e xs
	where
	(n,v)=x

data Token
      = TokenLet
      | TokenIn
      | TokenFloat Float
      | TokenString String
      | TokenVar String
      | TokenEq
      | TokenLT
      | TokenLEQ
      | TokenGT
      | TokenGEQ
      | TokenPlus
      | TokenAppend
      | TokenMinus
      | TokenTimes
      | TokenDiv
      | TokenOB
      | TokenCB
 deriving Show

tokenList :: String -> [Token]
tokenList xs = case xs of
	[] -> []
  	('\n':cs) -> tokenList cs
  	(c:cs) | isSpace c -> tokenList cs
	otherwise
		|token==[] -> []
		|token=="(" -> TokenOB : tokenList (tail xs)
		|token==")" -> TokenCB : tokenList (tail xs)
		|token=="=" -> TokenEq : tokenList (tail xs)
		|token=="<=" -> TokenLEQ : tokenList (tail (tail xs))
		|token=="<" -> TokenLT : tokenList (tail xs)
		|token==">=" -> TokenGEQ : tokenList (tail (tail xs))
		|token==">" -> TokenGT : tokenList (tail xs)
		|token=="+" -> TokenPlus : tokenList (tail xs)
		|token=="++" -> TokenAppend : tokenList (tail (tail xs))
		|token=="-" -> TokenMinus : tokenList (tail xs)
		|token=="*" -> TokenTimes : tokenList (tail xs)
		|token=="/" -> TokenDiv : tokenList (tail xs)
		|token=="let" -> TokenLet : tokenList (drop (length token) xs)
		|token=="in" -> TokenIn : tokenList (tail (tail xs))
		|isVar token -> (TokenVar token) : tokenList (drop (length token) xs)
		|isNum token -> (TokenFloat ((read token)::Float)) : tokenList (drop (length token) xs)
		|isText token -> (TokenString token) : tokenList (drop (length token) xs)
	where token = isToken xs

matchRe s re = wholeMatch( searchS re [] s)

isToken [] = []
isToken s = matchRe s "let|in|[a-zA-Z0-9\\.\\_]*|\\(|\\)|\\++|\\=|\\<=|\\<|\\>=|\\>|\\+|\\-|\\*|\\/"

isNum (x:xs) = (matchRe (x:xs) "[0-9\\.]+") == (x:xs)
isNum (_) = False

-- getNum s = wholeMatch( searchS "[0-9\\.]+" [] s)

isVar (x:xs) = isUpper x
isVar (_) = False

isText (x:xs) = True
isText (_) = False

evalExp::String->Bool
evalExp s = calc (tokenList s) []
}
